# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import driverng

parser_class = "bioplex_2200"


worklist = driverng.Worklist(
    context="result",
    samples=[
        {
            "sample_id": "123456",
            "type": "patient",
            "collection_date": datetime.datetime(2008, 1, 30, 8, 30),
            "request_date": datetime.datetime(2008, 1, 30, 8, 30),
            "last_name": "Last Name",
            "first_name": "First Name",
            "title": "M",
            "birthdate": datetime.datetime(1985, 12, 1),
            "sex": "M",
            "patient_id": "987654321",
            "priority": "R",
        }
    ],
)


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||BioPlex|||||||P||20081211113553
    Q|1|^123456||^^^ALL||||||||O
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist.samples[0].orders = [
        {"code": "ANAScr", "dilution": "", "action": "new", "priority": "A"}
    ]
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA|||||||P||20130715141912
    P|1|987654321|||Last Name^First Name^M||19851201|M
    O|1|123456||^^^ANAScr|R|20080130083000|20080130083000
    L|1|N
    """
    assert message == expected


def test_encode_order_with_dilution(self):
    worklist.samples[0].orders = [
        {"code": "EBVCAG", "dilution": "1:10", "action": "new", "priority": "A"}
    ]
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA|||||||P||20130715141912
    P|1|987654321|||Last Name^First Name^M||19851201|M
    O|1|123456||^^^EBVCAG^10^1|R|20080130083000|20080130083000
    L|1|N
    """
    assert message == expected


def test_encode_order_with_multiple_dilutions(self):
    worklist.samples[0].orders = [
        {"code": "dsdna", "dilution": "1;4;10;100", "action": "new", "priority": "A"}
    ]
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA|||||||P||20130715141912
    P|1|987654321|||Last Name^First Name^M||19851201|M
    O|1|123456||^^^dsdna^1^1|R|20080130083000|20080130083000
    O|2|123456||^^^dsdna^4^1|R|20080130083000|20080130083000
    O|3|123456||^^^dsdna^10^1|R|20080130083000|20080130083000
    O|4|123456||^^^dsdna^100^1|R|20080130083000|20080130083000
    L|1|N
    """
    assert message == expected


def test_encode_cancel_order(self):
    worklist.samples[0].orders = [
        {"code": "dsDNA-1", "dilution": "", "action": "del", "priority": "R"}
    ]
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA|||||||P||20130715141912
    P|1|987654321|||Last Name^First Name^M||19851201|M
    O|1|123456||^^^dsDNA-1|R|20080130083000|20080130083000||||C
    L|1|N
    """
    assert message == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&|||BioPlex|||||||P||20081211113553
            Q|1|^123456||^^^ALL||||||||O
            L|1|N
            """
            )
        },
    )
    expected = """
    H|\\^&|||MCA|||||||P||20130715141912
    P|1
    O|1|123456|||R||||||||||||||||||||Y
    L|1|N
    """
    assert message == expected


def test_decode_results(self):
    date = datetime.datetime(2010, 8, 3, 11, 19, 5)
    worklist = self.decode(
        """
    H|\\^&|||PU32|||||||P||20100803111903
    P|1|987654321|||Last Name^First Name^M||19851201|M
    O|1|BIO2010RAD^^PTRack1^1||^^^ANAScr|R|20100803111134|20080130083000
    R|1|^^^ANAScr^MDSSResult|POSITIVE|||||F|||20100803111856|20100803111905
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "PTRack1_1",
                "results": [
                    {
                        "code": "ANAScr_MDSSResult",
                        "comments": [],
                        "date": date,
                        "flags": [],
                        "unit": "",
                        "value": "POSITIVE",
                    }
                ],
                "sample_id": "BIO2010RAD",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_results_with_flags(self):
    date = datetime.datetime(2010, 6, 15, 10, 16, 52)
    worklist = self.decode(
        """
    H|\\^&|||PU32|||||||P||20100804082423
    P|1
    O|1|A0108^^Rack22^3||^^^MeaslG|R|20100615101213
    R|1|^^^MeaslG^QUANT|<0.2|AI||<^RV: -100000 Warning: SVB Too Low||F|||20100615101651|20100615101652
    R|2|^^^MeaslG^QUAL|Negative|||^RV: -100000 Warning: SVB Too Low||F|||20100615101651|20100615101652
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "Rack22_3",
                "results": [
                    {
                        "code": "MeaslG_QUANT",
                        "comments": [],
                        "date": date,
                        "flags": ["<", "RV: -100000 Warning: SVB Too Low"],
                        "unit": "AI",
                        "value": "<0.2",
                    },
                    {
                        "code": "MeaslG_QUAL",
                        "comments": [],
                        "date": date,
                        "flags": ["", "RV: -100000 Warning: SVB Too Low"],
                        "unit": "",
                        "value": "Negative",
                    },
                ],
                "sample_id": "A0108",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_results_with_comments(self):
    date = datetime.datetime(2010, 8, 3, 11, 54, 32)
    worklist = self.decode(
        """
    H|\\^&|||PU32|||||||P||20100803115616
    P|1|ANA Screen|||Last Name^First Name^M||19851201|M
    O|1|BIO2010-2RAD^^PTRack1^1||^^^ANA 11|R|20100803115322|20080130083000
    C|1|I|Antibody levels show association with MDSS profiles for systemic autoimmune disease. Consider SLE or Sjogren's syndrome. MDSS outputs of "Negative" or "No Association" do not rule out autoimmune disease. Patients with Rheumatoid Arthritis may result in an SLE association from MDSS, thus MDSS associations from patients with RA should be interpreted with caution.
    R|1|^^^ANA 11^|POSITIVE|||||F|||20100803115421|20100803115432
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "PTRack1_1",
                "results": [
                    {
                        "code": "ANA 11",
                        "comments": [
                            "Antibody levels show association with "
                            "MDSS profiles for systemic autoimmune "
                            "disease. Consider SLE or Sjogren's "
                            'syndrome. MDSS outputs of "Negative" '
                            'or "No Association" do not rule out '
                            "autoimmune disease. Patients with "
                            "Rheumatoid Arthritis may result in an "
                            "SLE association from MDSS, thus MDSS "
                            "associations from patients with RA "
                            "should be interpreted with caution."
                        ],
                        "date": date,
                        "flags": [],
                        "unit": "",
                        "value": "POSITIVE",
                    }
                ],
                "sample_id": "BIO2010-2RAD",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2010, 8, 4, 11, 33, 5)
    worklist = self.decode(
        """
    H|\\^&|||PU32|||||||P||20100804113311
    P|1
    O|1|dsDNA^Level 1^QCRack1^1|46416|^^^dsDNA||20100804113303|||||Q
    R|1|^^^dsDNA^QUANT|0|IU/mL||<||F|||20100804113304|20100804113305
    R|2|^^^dsDNA^QUAL|Negative|||||F|||20100804113304|20100804113305
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "46416",
                "results": [
                    {
                        "code": "dsDNA_QUANT",
                        "comments": [],
                        "date": date,
                        "flags": ["<"],
                        "unit": "IU/mL",
                        "value": "0",
                    },
                    {
                        "code": "dsDNA_QUAL",
                        "comments": [],
                        "date": date,
                        "flags": [],
                        "unit": "",
                        "value": "Negative",
                    },
                ],
                "sample_id": "dsDNA_Level 1",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
