# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

import driverng
from driverng.tests import BaseTest


class Pentra120Test(BaseTest):
    MAPPING_CLASS = "pentra120"

    @staticmethod
    def unindent(data):
        return "\r".join([li.lstrip() for li in data.splitlines() if li.strip()])

    def test_analysis_date(self):
        date = datetime.datetime(2013, 2, 20)
        Pentra120Message = self.F.__class__
        for parser, expected_string in (
            (Pentra120Message(), "q 20/02/13 00h00mn00s"),
            (
                Pentra120Message(format_analysis_date="%y/%m/%d %Hh%Mmn%Ss"),
                "q 13/02/20 00h00mn00s",
            ),
        ):
            obj = parser.create()
            obj.analysis_date = date
            raw = parser.to_string(obj)
            for line in raw.split("\r"):
                if not line:
                    continue
                abx_id = ord(line[0])
                if abx_id == 0x71:
                    break
            else:
                raise Exception("Date non trouvée")
            self.assertEqual(line, expected_string)
            self.assertEqual(parser.parse(raw).analysis_date, date)

    def test_order_message(self):
        worklist = self.get_example_worklist([{"code": "CBC", "action": "new"}])
        expected = (
            "\xff FILE    \r"
            "u 123456          \r"
            "v NYME ANO                      \r"
            "w 11/05/13\r"
            "x 2y \r"
            "} 13/05/13 11h42\r"
            "\x7f Serum           \r"
            "\x80 A\r"
        )
        self.assertOrderMessageEqual(worklist, expected)

    def test_invalid_order(self):
        with self.assertRaises(RuntimeError):
            self.F.get_order_message(
                self.get_example_worklist(
                    [{"code": "DIF", "action": "new"}, {"code": "ERB", "action": "new"}]
                ),
                {},
            )

    def test_order_no_birthdate(self):
        wk = self.get_example_worklist([{"code": "CBC", "action": "new"}])
        wk.samples[0]["birthdate"] = None
        self.assertOrderMessageEqual(
            wk,
            (
                "\xff FILE    \r"
                "u 123456          \r"
                "v NYME ANO                      \r"
                "} 13/05/13 11h42\r"
                "\x7f Serum           \r"
                "\x80 A"
            ),
        )

    def test_qc(self):
        date = datetime.datetime(2018, 12, 13, 13, 32, 29)
        self.assertQCEqual(
            (
                "\xff QC-RES-H\r"
                "! 017.6  \r"
                '" 02.59  \r'
                "# 014.7  \r"
                "$ 01.09  \r"
                "% 006.2  \r"
                "( 012.7  \r"
                ") 072.3  \r"
                "* 00.74  \r"
                "+ 004.2  \r"
                ", 00.46  \r"
                "- 002.6  \r"
                ". 00.09 H\r"
                "/ 000.5 H\r"
                "0 01.40 H\r"
                "1 008.6 H\r"
                "2 05.22  \r"
                "3 16.14  \r"
                "4 046.2  \r"
                "5 00089  \r"
                "6 30.94  \r"
                "7 34.91  \r"
                "8 013.5  \r"
                "@ 00550  \r"
                "A 008.6  \r"
                "B 0.471 H\r"
                "C 016.0 H\r"
                "P             \r"
                "Q                          \r"
                "R     \r"
                "S       \r"
                "f               \r"
                "p 01\r"
                "q 13/12/18 13h32mn29s\r"
                "s 0120\r"
                "t M\r"
                "u PX113H          \r"
                "\x80 B\r"
                "\x82 1\r"
                "\x83 ta \r"
                "\xfb HDG02   \r"
                "\xfe V5.0 "
            ),
            [
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "017.6",
                    "test": "21",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "02.59",
                    "test": "22",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "014.7",
                    "test": "23",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "01.09",
                    "test": "24",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "006.2",
                    "test": "25",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "012.7",
                    "test": "28",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "072.3",
                    "test": "29",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "00.74",
                    "test": "2a",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "004.2",
                    "test": "2b",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "00.46",
                    "test": "2c",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "002.6",
                    "test": "2d",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "00.09",
                    "test": "2e",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "000.5",
                    "test": "2f",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "01.40",
                    "test": "30",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "008.6",
                    "test": "31",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "05.22",
                    "test": "32",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "16.14",
                    "test": "33",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "046.2",
                    "test": "34",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "00089",
                    "test": "35",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "30.94",
                    "test": "36",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "34.91",
                    "test": "37",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "013.5",
                    "test": "38",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "00550",
                    "test": "40",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "008.6",
                    "test": "41",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "0.471",
                    "test": "42",
                },
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "016.0",
                    "test": "43",
                },
            ],
        )

    def test_result(self):
        date = datetime.datetime(2018, 12, 13, 13, 41, 51)
        self.assertResultEqual(
            (
                "\xff RESULT  \r"
                "! 006.1 A\r"
                '" 01.72  \r'
                "# 028.1  \r"
                "$ 00.63  \r"
                "% 010.3  \r"
                "( 03.55  \r"
                ") 058.1  \r"
                "* 00.19  \r"
                "+ 003.1  \r"
                ", 00.02  \r"
                "- 000.4  \r"
                ". 00.09  \r"
                "/ 001.5  \r"
                "0 00.05  \r"
                "1 000.9  \r"
                "2 04.70  \r"
                "3 14.68  \r"
                "4 043.9  \r"
                "5 00093  \r"
                "6 31.26  \r"
                "7 33.44  \r"
                "8 012.7  \r"
                "@ 00170  \r"
                "A 009.2  \r"
                "B 0.157  \r"
                "C 017.5  \r"
                "Q              MNLNLI1  \r"
                "V PLAG MAPL\r"
                "p 01\r"
                "q 13/12/18 13h41mn51s\r"
                "r 0002            \r"
                "s 0208\r"
                "t M\r"
                "u 420112188366    \r"
                "}               \r"
                "~                                 \r"
                "\x7f standard        \r"
                "\x80 B\r"
                "\x82 1\r"
                "\x83 ta \r"
                "\xb4 000.1  \r"
                "\xb5 00.01  \r"
                "\xb6 000.4  \r"
                "\xb7 00.02  \r"
                "\xb8 000.4  \r"
                "\xb9 00.02  \r"
                "\xc8 022 025 048 037 106 030 067 068 088 127 145 190 031 068 050 021 026 028 055 031 070 037 095 002 002 002\r"
                "\xfb HDG02   \r"
                "\xfe V5.0 "
            ),
            {
                "420112188366": {
                    "rack": "0002",
                    "resultats": [
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "006.1",
                            "test": "21",
                            "operator": "ta",
                            "alarmes": ["A"],
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "01.72",
                            "test": "22",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "028.1",
                            "test": "23",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.63",
                            "test": "24",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "010.3",
                            "test": "25",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "03.55",
                            "test": "28",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "058.1",
                            "test": "29",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.19",
                            "test": "2a",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "003.1",
                            "test": "2b",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.02",
                            "test": "2c",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "000.4",
                            "test": "2d",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.09",
                            "test": "2e",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "001.5",
                            "test": "2f",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.05",
                            "test": "30",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "000.9",
                            "test": "31",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "04.70",
                            "test": "32",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "14.68",
                            "test": "33",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "043.9",
                            "test": "34",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00093",
                            "test": "35",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "31.26",
                            "test": "36",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "33.44",
                            "test": "37",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "012.7",
                            "test": "38",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "000.1",
                            "test": "b4",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.01",
                            "test": "b5",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00170",
                            "test": "40",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "009.2",
                            "test": "41",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "0.157",
                            "test": "42",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "017.5",
                            "test": "43",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "000.4",
                            "test": "b6",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.02",
                            "test": "b7",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "000.4",
                            "test": "b8",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "00.02",
                            "test": "b9",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "LI1;LN;MAPL;MN;PLAG",
                            "test": "ALARM",
                            "operator": "ta",
                        },
                        {
                            "analyseur": "HDG0201",
                            "date": date,
                            "resu": "",
                            "test": "ALARME",
                            "commentaires": ["LI1", "LN", "MAPL", "MN", "PLAG"],
                            "operator": "ta",
                        },
                    ],
                }
            },
        )

    def test_query(self):
        self.assertQueryEqual(
            (
                "\xff FILE    \r"
                "u 460112188417    \r"
                "w 19600812\r"
                "x 53y\r"
                "} 18/12/13 08h11\r"
                "\x80 A\r"
            ),
            ["460112188417"],
        )

    def test_qc_prefix(self):
        date = datetime.datetime(2018, 12, 13, 13, 32, 29)
        self._F = driverng.get(self.MAPPING_CLASS)(qc_prefix="PX")
        self.assertQCEqual(
            (
                "\xff RESULT\r"
                "! 017.6  \r"
                "p 01\r"
                "q 13/12/18 13h32mn29s\r"
                "s 0120\r"
                "t M\r"
                "u PX113H          \r"
                "\x80 B\r"
                "\x82 1\r"
                "\x83 ta \r"
                "\xfb HDG02   \r"
                "\xfe V5.0 "
            ),
            [
                {
                    "analyseur": "HDG0201",
                    "lot": "PX113H",
                    "date": date,
                    "resu": "017.6",
                    "test": "21",
                }
            ],
        )
